#include "platform_api.h"
#include <msp/strings/format.h>
#include "inet.h"
#include "sockaddr_private.h"

using namespace std;

namespace Msp {
namespace Net {

InetAddr::InetAddr():
	port(0)
{
	fill(addr, addr+4, 0);
}

InetAddr::InetAddr(const SysAddr &sa)
{
	const sockaddr_in &sai = reinterpret_cast<const sockaddr_in &>(sa.addr);
	const char *ptr = reinterpret_cast<const char *>(&sai.sin_addr.s_addr);
	std::copy(ptr, ptr+4, addr);
	port = ntohs(sai.sin_port);
}

SockAddr::SysAddr InetAddr::to_sys() const
{
	SysAddr sa;
	sa.size = sizeof(sockaddr_in);
	sockaddr_in &sai = reinterpret_cast<sockaddr_in &>(sa.addr);
	sai.sin_family = AF_INET;
	char *ptr = reinterpret_cast<char *>(&sai.sin_addr.s_addr);
	std::copy(addr, addr+4, ptr);
	sai.sin_port = htons(port);

	return sa;
}

string InetAddr::str() const
{
	string result = format("%d.%d.%d.%d", addr[0], addr[1], addr[2], addr[3]);
	if(port)
		result += format(":%d", port);
	return result;
}

} // namespace Net
} // namespace Msp
