#ifndef MSP_NET_SOCKET_H_
#define MSP_NET_SOCKET_H_

#include <msp/io/eventobject.h>
#include <msp/io/handle.h>
#include "constants.h"
#include "sockaddr.h"

namespace Msp {
namespace Net {

class bad_socket_state: public std::logic_error
{
public:
	bad_socket_state(const std::string &w): std::logic_error(w) { }
	virtual ~bad_socket_state() throw() { }
};


class Socket: public IO::EventObject
{
protected:
	enum SocketEvent
	{
		S_NONE = IO::P_NONE,
		S_INPUT = IO::P_INPUT,
		S_CONNECT = 4096,
		S_ACCEPT = 8192
	};

	struct Private;

	Private *priv;
	SockAddr *local_addr;

	Socket(const Private &);
	Socket(Family, int, int);
private:
	void platform_init();
	void platform_cleanup();
public:
	~Socket();

	virtual void set_block(bool);
	virtual const IO::Handle &get_event_handle();

	/** Associates the socket with a local address.  There must be no existing
	users of the address. */
	void bind(const SockAddr &);

	const SockAddr &get_local_address() const;

	void set_timeout(const Time::TimeDelta &);
protected:
	void set_socket_events(unsigned);
	void set_platform_events(unsigned);
};

} // namespace Net
} // namespace Msp

#endif
